/*
 * name ProtoSnap_Pro_Mini.pde
 * date Sat Mar 09 20:23:25 JST 2013
 * 
 * Released under the 'Buy Me a Beer' license
 * (If we ever meet, you buy me a beer)
 */

import net.triring.emuduino.*;
import ddf.minim.AudioOutput;
// Pin割り当て
// constants won't change. They're used here to 
// set pin numbers:
int      red =  3;  // Led:Red
int    green =  5;  // Led:Green
int     blue =  6;  // Led:Blue
int   button =  7;  // Switch:Push
int  ard_led = 13;  // Led:Arduino
int   buzzer = 10;  // Speaker:
// int   buzzer =  2;  // Speaker:
int    light =  0;  // Slider:INPUT

int       S0 =  0;  // Serial
int       S1 =  1;  // Serial

int   n = 0;
int   r = 0;
int   g = 0;
int   b = 0;

/*
+--------------+------------+
|Component Pin | Arduino Pin|
+--------------+------------+
| Button       |      7     |
+--------------+------------+
| Light Sensor |     A0     |
+--------------+------------+
| Green LED    |      5     |
+--------------+------------+
| Blue LED     |      6     |
+--------------+------------+
| Red LED      |      3     |
+--------------+------------+
| Buzzer       |      2     |
+--------------+------------+
*/

int REST =    0; // 休符
int O4C  =  264; // ド    264 Hz
int O4CS =  282; //       282 Hz
int O4D  =  297; // レ    297 Hz
int O4DS =  312; //       312 Hz
int O4E  =  330; // ミ    330 Hz
int O4F  =  352; // ファ  352 Hz
int O4FS =  371; //       371 Hz
int O4G  =  396; // ソ    396 Hz
int O4GS =  422; //       422 Hz
int O4A  =  440; // ラ    440 Hz
int O4AS =  475; //       475 Hz
int O4B  =  495; // シ    495 Hz
int O5C  =  528; // ド    528 Hz
int O5CS =  563; //       563 Hz
int O5D  =  594; // レ    594 Hz
int O5DS =  624; //       624 Hz
int O5E  =  660; // ミ    660 Hz
int O5F  =  704; // ファ  704 Hz
int O5FS =  743; //       743 Hz
int O5G  =  792; // ソ    792 Hz
int O5GS =  845; //       845 Hz
int O5A  =  880; // ラ    880 Hz
int O5AS =  950; //       950 Hz
int O5B  =  990; // シ    990 Hz


/* 4分音符 = 480Tick として設定          */
int L1_DOT  = 255;   //       全音符 12
int L1      = 192;   //       全音符 11
int L2_DOT  = 144;   // 付点 2分音符 10
int L2      =  96;   //      2分音符  9
int L4_DOT  =  72;   // 付点 4分音符  8
int L4      =  48;   //      4分音符  7
int L8_DOT  =  36;   // 付点 8分音符  6
int L8      =  24;   //      8分音符  5
int L16_DOT =  18;   // 付点16分音符  4
int L16     =  12;   //     16分音符  3
int L32_DOT =   9;   // 付点32分音符  2
int L32     =   6;   //     32分音符  1

float beatsPerMinute = 83;
// ファミマ入店音 コンビニ入店時チャイム
int[] famimaNotes     = { O5G, O5DS, O4AS, O5DS, O5F, O5AS,   O5F, O5G, O5F, O4AS, O5DS, O5DS,   REST };
int[] famimaDurations = { L8,  L8,   L8,   L8,   L8,  L4_DOT, L8,  L8,  L8,  L8,   L1,   L4_DOT, L8   };
int   famimaLength = 12; // データ数

int thisNote = 0;
int noteDuration;
int pauseBetweenNotes;


void setup() {
/*
ProtoSnap - Pro Mini
SparkFun社のProtoSnap - Pro Miniをエミュレートしました。
このボードには，Arduino Pro miniとタクトスイッチ、光センサー、フルカラーLED、ブザーがセットになっています。
lightセンサの値が128以上になると，LEDが点滅します。
スイッチが押されている間は，音楽が鳴り続けます。
商品コード： SFE-DEV-10889
https://www.sparkfun.com/products/10889
*/
  initComponents();
  // initialize
  Serial.begin(9600);
  pinMode(red,       OUTPUT);  // Led:Red
  pinMode(green,     OUTPUT);  // Led:Green
  pinMode(blue,      OUTPUT);  // Led:Blue
  pinMode(button,    INPUT );  // Switch:Push
  pinMode(buzzer,    OUTPUT);  // Speaker:
  pinMode(ard_led, OUTPUT);  // Led:Arduino
  Serial.println("I am a protosnap, waiting to hear space bar:");
}

void loop() {
  if(Serial.available() > 0) {
    if(Serial.read() == 32)
      Serial.println("I HEARD THAT, OH SNAP!");
  }

  if (digitalRead(button) == HIGH) {
    noteDuration = (int)((60000/beatsPerMinute)*famimaDurations[thisNote] / 96);
    if (REST != famimaNotes[thisNote]) {
      tone(buzzer, famimaNotes[thisNote],noteDuration);
       Serial.print(thisNote);
       Serial.print(",");
       Serial.print(famimaNotes[thisNote]);
       Serial.print(",");
       Serial.print(noteDuration);
       Serial.print(",");
    }
    pauseBetweenNotes = (int)(noteDuration * 1.20);
    delay(pauseBetweenNotes);
    thisNote++;
    if (thisNote > famimaLength) thisNote = 0;
  } else {
    delay(500);
  }

  int temp = analogRead(light);
  Serial.print("light=");
  Serial.println("" + temp);

  if (128 > temp) {
    r = LOW ; g = LOW ; b = LOW ;
  } else {
    switch(n) {
      case  0 : r = LOW ; g = LOW ; b = LOW ;        break; 
      case  1 : r = HIGH; g = LOW ; b = LOW ;        break; 
      case  2 : r = LOW ; g = HIGH; b = LOW ;        break; 
      case  3 : r = HIGH; g = HIGH; b = LOW ;        break; 
      case  4 : r = LOW ; g = LOW ; b = HIGH;        break; 
      case  5 : r = HIGH; g = LOW ; b = HIGH;        break; 
      case  6 : r = LOW ; g = HIGH; b = HIGH;        break; 
      case  7 : r = HIGH; g = HIGH; b = HIGH; n = 0; break; 
    }
    n++;
    if (0 == (n % 2)) {
      digitalWrite(ard_led,   HIGH);
    } else {
      digitalWrite(ard_led,   LOW );
    }
    digitalWrite(red,   r);
    digitalWrite(green, g);
    digitalWrite(blue,  b);
  }
}

void draw() {
  loop();
}

// システムの機能を完全に停止させる禁断のメソッド
// 決して、使ってはならない。
void balse() {
  System.exit(0);
}                                            

// digital INPUT
int digitalRead(int inPin) {
    int value = 0;
    switch (inPin) {
        case 7 : value = switch7.isState(); break;
    }
    return value;
}
// digital OUTPUT
void digitalWrite(int outPin, int value) {
    switch (outPin) {
        case  3 :      led3.setState(value); break;
        case  5 :      led5.setState(value); break;
        case  6 :      led6.setState(value); break;
        case 13 : arduino13.setState(value); break;
    }
}
// analog INPUT
int analogRead(int inPin) {
    int value = 0;
    switch (inPin) {
        case   0 : value = slider0.getValue(); break;
    }
    return value;
}
// Speakerで使用する関数群

void tone(int pin, int frequency) {
  if (ToneFlag == frequency) return;
  ToneFlag = frequency;
  speaker10.setFrequency(frequency);
  sine.setFreq((float)frequency);
  auOut.unmute();
  auOut.sound();
}

void tone(int pin, int frequency, long duration) {
  if (ToneFlag == frequency) return;
  ToneFlag = frequency;
  sine.setFreq((float)frequency);
  auOut.unmute();
  auOut.sound();
  TimerTaskAudioStop timerTask = new TimerTaskAudioStop(); //タイマータスクインスタンス作成
  //タイマー開始
  DelayTimer.schedule(timerTask, duration);
}

void noTone() {
  ToneFlag = 0;
  speaker10.setFrequency(0);
  auOut.mute();
  auOut.noSound();
}
java.awt.Image readImage(String imgfile) {
  PImage pImage = loadImage(imgfile);
  return pImage.getImage();
}

//TimerTask内部クラス
class TimerTaskAudioStop extends java.util.TimerTask {
  public void run() {
    ToneFlag = 0;
    auOut.noSound();
    auOut.mute();
  }
}

// 指定された時間だけ停止する。単位はus
void delayMicroseconds(int t_us) {
  try {
    Thread.sleep(0, t_us * 1000);  // マイクロ秒単位の停止
  } 
  catch (InterruptedException e) {
  }
}

// 指定された時間だけ停止する。単位はms
void delay(long t_ms) {
  try {
    Thread.sleep(t_ms);  // ミリ秒単位の停止
  } 
  catch (InterruptedException e) {
  }
}

// arduino起動後の時間をus単位で返す。
long micros() {
  return (SystemStartTimeNS - System.nanoTime()) * 1000;
}

// 以下は、互換性のためだけに用意したダミー関数である。
void pinMode(int pin, int mode) {
   print("pin = " + pin + ", mode = " + mode);
}

/**
 * This method is called from within the constructor to initialize the form.
 * WARNING: Do NOT modify this code. The content of this method is always
 * regenerated by the Form Editor.
 */
// @SuppressWarnings("unchecked")
// <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
private void initComponents() {
  ToneFlag = 0;
  // digital
  // arduino13 = new Arduino(13 , "arduino13",   0,    0, "controller_arduino_diecimila");
     arduino13 = new Arduino(13 , "arduino13",   0,    0, "controller_arduino_pro_mini_red");
     speaker10 = new Speaker(10 , "speaker10", 164,    0);
          led3 = new Led(     3 , "red",       314,    0);
          led5 = new Led(     5 , "green",     314,  144);
          led6 = new Led(     6 , "blue",      314,  288);
       slider0 = new Slider(  0 , "INPUT0",    434,    0);
       switch7 = new Switch(  7 , "Push7",     554,    0);

        Serial = new SerialConsole(434, 288);

  minim = new ddf.minim.Minim(this);
  // get a line out from Minim, default bufferSize is 1024, default sample rate is 44100, bit depth is 16
  auOut = minim.getLineOut(ddf.minim.Minim.STEREO);
  // create a sine wave Oscillator, set to 440 Hz, at 0.5 amplitude, sample rate from line out
  sine = new ddf.minim.signals.SineWave(440, 0.5, auOut.sampleRate());
  // set the portamento speed on the oscillator to 200 milliseconds
  sine.portamento(200);

  DelayTimer = new java.util.Timer(); //タイマーインスタンス作成

  // add the oscillator to the line out
  auOut.addSignal(sine);
  auOut.noSound();  // Disables all signals currently attached to this.
  auOut.mute();
  ToneFlag = 0;

  SystemStartTimeMS = System.currentTimeMillis();  //ミリ秒で表される現在の時間を返します。
  SystemStartTimeNS = System.nanoTime(); // 利用可能でもっとも正確なシステムタイマーの現在の値をナノ秒単位で返します。
  size(320, 128);
  this.setLocation(0, 512);
  background(15, 115, 145);
  fill( 15, 115, 145);  rect(0, 0, 320, 128);
  fill( 30, 160, 145);  textSize(60); text(TITLE , 24, 68);
  fill( 45, 192, 145);  textSize(60); text(TITLE , 22, 66);
  fill(255, 255, 255);  textSize(60); text(TITLE , 20, 64);
  fill(255, 255, 255);  textSize(18); text("Ver. " + Arduino.getVersion(), 120, 108);
  this.setLocation(0, 512);
}// </editor-fold>//GEN-END:initComponents

// 変数の宣言部
// Variables declaration - do not modify//GEN-BEGIN:variables

private String TITLE   = "Emuduino";
private String VERSION = "0.49"; // Arduino.javaで使用するプロパティファイル中のバージョンを参照する。

private long SystemStartTimeMS;
private long SystemStartTimeNS;

private static final int HIGH   = 0x1;
private static final int LOW    = 0x0;
private static final int INPUT  = 0x0;
private static final int OUTPUT = 0x1;

private int ToneFlag = 0;  // 0の場合は停止状態。再生中は，その周波数を保持

private java.util.Timer            DelayTimer;
private java.util.TimerTask        TimerTaskAudioStop;
private ddf.minim.Minim            minim; // http://code.compartmental.net/minim/javadoc/
private ddf.minim.AudioOutput      auOut;
private ddf.minim.signals.SineWave sine;

private Led led3;
private Led led5;
private Led led6;
private Switch switch7;
private Speaker speaker10;
private Arduino arduino13;
private Slider slider0;
private SerialConsole Serial;


// End of variables declaration//GEN-END:variables


